import json

import pytest

from aad_fastapi import AzureAdSettings
from tests.helpers import gen_payload


@pytest.mark.parametrize(
    "roles, expected_status_code",
    [
        (["Admin", "Contributor"], 200),
        (["Admin"], 403),
        (["Contributor"], 403),
        ([], 403),
    ],
)
def test_isauth_with_impersonation_and_all_roles(
    mock_test_client, private_key, roles, expected_status_code
):
    options = AzureAdSettings()
    payload = gen_payload(options, private_key, roles=roles, scp=["user_impersonation"])
    token = json.loads(payload)["access_token"]
    response = mock_test_client.get(
        "/isauth_impersonation_all_roles", headers={"Authorization": f"Bearer {token}"}
    )
    assert response.status_code == expected_status_code


@pytest.mark.parametrize(
    "roles,expected_status_code",
    [
        (["Admin", "Contributor"], 200),
        (["Admin"], 200),
        (["Contributor"], 200),
        ([], 403),
    ],
)
def test_valid_access_token_with_any_roles(
    mock_test_client, private_key, roles, expected_status_code
):
    options = AzureAdSettings()
    payload = gen_payload(options, private_key, roles=roles, scp=["user_impersonation"])
    token = json.loads(payload)["access_token"]
    response = mock_test_client.get(
        "/isauth_impersonation_any_roles", headers={"Authorization": f"Bearer {token}"}
    )
    assert response.status_code == expected_status_code



from typing import Optional


class AuthError(Exception):
    code: str
    description: str

    def __init__(
        self,
        code: Optional[str] = None,
        description: Optional[str] = None,
        status_code=401,
        exception: Optional[Exception] = None,
    ):
        self.populate_code(code, exception)
        self.populate_description(description, exception)
        super().__init__(self.description)
        self.status_code = status_code

    def populate_code(self, code, exception):
        if code is None and exception is not None:
            if hasattr(exception, "error"):
                code = exception.error
            elif hasattr(exception, "code"):
                code = exception.code
        self.code = code

    def populate_description(self, description, exception):
        if description is None and exception is not None:
            if hasattr(exception, "message"):
                description = exception.message
            elif hasattr(exception, "description"):
                description = exception.description
            elif hasattr(exception, "args") and len(exception.args) >= 1:
                description = exception.args[0]
        self.description = description



import typing


class RoleValidatorInterface:
    def validate_roles(
        self, user_roles: typing.Sequence[str], mandatory_roles: typing.Sequence[str]
    ) -> bool:
        """validate the user roles against the mandatory roles"""
